module fitting_module
integer,parameter :: maxdata=1000
real*8 x(maxdata),refval(maxdata) !S66 dimer index, disp/elst ratio in S66 original paper
real*8 DC(maxdata) !Dispersion correction term
real*8 DFTc(maxdata) !Correlation functional
real*8 elst(maxdata) !Electrostatic term
end module
    
program LMfit
use fitting_module
implicit real*8 (a-h,o-z)
integer,parameter :: nparm=3
real*8 parm(nparm)
real*8 fiterr(maxdata),fitval(maxdata)
character c80tmp*80,c80tmp2*80,file1*200,file2*200
external calcfiterr

call getarg(1,file1)
call getarg(2,file2)
if (trim(file1)=="") file1="S66_de.txt"
if (trim(file2)=="") file2="data.txt"
open(10,file=file1,status="old")
open(11,file=file2,status="old")
ndata=0
do while(.true.)
    read(10,"(a)",iostat=ierror) c80tmp
    read(11,"(a)",iostat=ierror) c80tmp2
    if (c80tmp==" ".or.ierror/=0) exit
    read(c80tmp,*) itmp,tmp
    if (tmp>3.8) cycle
    ndata=ndata+1
    read(c80tmp,*) x(ndata),refval(ndata)
    read(c80tmp2,*) idx,etot,elst(ndata),orb,DFTc(ndata),DC(ndata)
end do
close(10)
close(11)
write(*,"(i6,' data have been loaded')") ndata
write(*,*) "Index     S66 d/e     E_elst     E_DFTc     E_disp.corr."
do idata=1,ndata
    write(*,"(i3,4f14.6)") idata,refval(idata),elst(idata),DFTc(idata),DC(idata)
end do

tol=1D-6
maxcall=10000
!Initial values
!parm(1)=0.5 !c
!parm(2)=0.0 !a
!parm(3)=1.0 !r

parm(1)=0.5 !c
parm(2)=0 !a
parm(3)=2 !r
write(*,*)
write(*,*) "Fitting via Levenberg-Marquardt algorithm..."
call lmdif1(calcfiterr,ndata,nparm,parm(:),fiterr(1:ndata),tol,maxcall,info)

if (info==1.or.info==2.or.info==3) then
    write(*,*) "Fitting has successfully finished!"
else if (info==5) then
    write(*,"(a,i7)") " Warning: Convergence tolerance has not met while the maximum number of function calls has reached",maxcall
else if (info==6.or.info==7) then
    write(*,*) "Error: Tolerance is too small, unable to reach the tolerance!"
end if

write(*,"(' Fitted parameters:')")
!do i=1,nparm
!    write(*,"(i5,f12.6)") i,parm(i)
!end do
write(*,"(' c:',f12.6)") parm(1)
write(*,"(' a:',f12.6)") parm(2)
write(*,"(' r:',f12.6)") parm(3)
write(*,"(' RMSE:',f12.4)") dsqrt(sum(fiterr(1:ndata)**2)/ndata)
write(*,"(' RRMSE:',f12.4)") dsqrt(sum(fiterr(1:ndata)**2)/sum(refval(1:ndata)**2))
write(*,"(' MAPE:',f12.4,' %')") sum(abs(fiterr(1:ndata)/refval(1:ndata)))/ndata*100

call calcfitval(ndata,nparm,parm,fitval(1:ndata))
pearsoncoeff=covarray(refval(1:ndata),fitval(1:ndata))/stddevarray(refval(1:ndata))/stddevarray(fitval(1:ndata))
write(*,"(2(a,f12.6),/)") " Pearson correlation coefficient r:",pearsoncoeff,"  r^2:",pearsoncoeff**2

absremax=0
do idata=1,ndata
    re=(fitval(idata)-refval(idata))/refval(idata)*100
    absre=abs(re)
    write(*,"(' Index=',i4,'  Ref.=',f9.3,'  Fitted=',f9.3,'  Relative error=',f9.2,' %')") idata,refval(idata),fitval(idata),re
    if (absre>absremax) then
        absremax=absre
        iabsremax=idata
    end if
end do
write(*,"(' MaxAPE:',f10.3,' % at index',i5)") absremax,iabsremax

!write(*,*)
!write(*,*) "Press ENTER button to exit"
!read(*,*)

contains

real*8 function stddevarray(array)
real*8 array(:),avg
avg=sum(array)/size(array)
stddevarray=dsqrt(sum((array-avg)**2)/size(array))
end function

real*8 function covarray(array1,array2)
real*8 array1(:),array2(:),avg1,avg2
avg1=sum(array1)/size(array1)
avg2=sum(array2)/size(array2)
covarray=sum((array1-avg1)*(array2-avg2))/size(array1)
end function

end program


!------ The routine calculates fitting error
subroutine calcfiterr(ndata,nparm,parm,fiterr,iflag)
use fitting_module
implicit real*8 (a-h,o-z)
integer ndata,nparm,iflag
real*8 :: parm(nparm),fiterr(ndata),fitval(ndata)

call calcfitval(ndata,nparm,parm,fitval)

do idata=1,ndata
    fiterr(idata)=abs(fitval(idata)-refval(idata))/refval(idata)
end do
end subroutine

!------ The routine calculates fitted function at loaded points
subroutine calcfitval(ndata,nparm,parm,fitval)
use fitting_module
implicit real*8 (a-h,o-z)
integer ndata,nparm
real*8 :: parm(nparm),fitval(ndata)
c=parm(1)
!c=0.5
a=parm(2)
!a=-0.2
r=parm(3)
!r=0.5
do idata=1,ndata
    DCelst=DC(idata)/elst(idata)
    !Full form using erfc: erfc(4/r*(x-(a+r/2)))/(2/k)+c
    !scale=erfc(4/r*(DCelst-(a+r/2))) / (2/(1-c)) + c
    !Full form using exponent function: exp(-r*(x-a))*(1-c)+c
    scale=exp(-r*(DCelst-a))*(1-c)+c
    
    fitval(idata) = (DFTc(idata)*scale + DC(idata))/elst(idata)
end do
end subroutine

